@@ -4,13 +4,13 @@
 #undef PLUGINSDIR
 
 /* misc.. */
-#undef MEM_DEBUG
 #undef HAVE_IPV6
 #undef HAVE_POPT_H
 #undef HAVE_SOCKS_H
 #undef HAVE_PL_PERL
 #undef HAVE_STATIC_PERL
 #undef HAVE_GMODULE
+#undef WANT_BIG5
 
 /* macros/curses checks */
 #undef HAS_CURSES
@@ -22,12 +22,13 @@
 #undef SCO_FLAVOR
 
 /* our own curses checks */
-#undef USE_CURSES_WINDOWS
 #undef HAVE_NCURSES_USE_DEFAULT_COLORS
 #undef HAVE_CURSES_IDCOK
 #undef HAVE_CURSES_RESIZETERM
 #undef HAVE_CURSES_WRESIZE
-#undef USE_CURSES_WINDOWS
+
+/* terminfo/termcap */
+#undef HAVE_TERMINFO
 
 /* nls */
 #undef ENABLE_NLS
@@ -35,3 +36,6 @@
 #undef HAVE_GETTEXT
 #undef HAVE_LC_MESSAGES
 #undef HAVE_STPCPY
+
+/* SSL */
+#undef HAVE_OPENSSL
