@@ -23,18 +23,20 @@
 #include "signals.h"
 #include "misc.h"
 #include "settings.h"
-
-#include "irc.h"
 #include "levels.h"
-#include "servers.h"
+
+#include "irc-servers.h"
 #include "irc-channels.h"
 #include "nicklist.h"
+#include "mode-lists.h"
 
 #include "../core/module-formats.h"
 #include "printtext.h"
 #include "fe-channels.h"
+#include "fe-irc-server.h"
 
-static void event_received(IRC_SERVER_REC *server, const char *data);
+static void print_event_received(IRC_SERVER_REC *server, const char *data,
+				 const char *nick, int target_param);
 
 static char *last_away_nick = NULL;
 static char *last_away_msg = NULL;
@@ -66,48 +68,49 @@ static void event_ison(IRC_SERVER_REC *server, const char *data)
 
 static void event_names_list(IRC_SERVER_REC *server, const char *data)
 {
+	IRC_CHANNEL_REC *chanrec;
 	char *params, *channel, *names;
 
 	g_return_if_fail(data != NULL);
 
 	params = event_get_params(data, 4, NULL, NULL, &channel, &names);
-	if (irc_channel_find(server, channel) == NULL) {
+
+	chanrec = irc_channel_find(server, channel);
+	if (chanrec == NULL || chanrec->names_got) {
 		printformat_module("fe-common/core", server, channel,
 				   MSGLEVEL_CRAP, TXT_NAMES,
-				   channel, names);
+				   channel, 0, 0, 0, 0, 0);
+                printtext(server, channel, MSGLEVEL_CRAP, "%s", names);
+
 	}
 	g_free(params);
 }
 
-static void event_end_of_names(IRC_SERVER_REC *server, const char *data)
+static void event_end_of_names(IRC_SERVER_REC *server, const char *data,
+			       const char *nick)
 {
-	char *params, *channel;
 	IRC_CHANNEL_REC *chanrec;
+	char *params, *channel;
 
 	g_return_if_fail(data != NULL);
 
 	params = event_get_params(data, 2, NULL, &channel);
 
 	chanrec = irc_channel_find(server, channel);
-	if (chanrec != NULL)
-		fe_channels_nicklist(CHANNEL(chanrec),
-				     CHANNEL_NICKLIST_FLAG_ALL);
-	else {
-		printformat_module("fe-common/core", server, channel,
-				   MSGLEVEL_CRAP, TXT_ENDOFNAMES,
-				   channel, 0, 0, 0, 0);
-	}
+	if (chanrec == NULL || chanrec->names_got)
+		print_event_received(server, data, nick, FALSE);
 	g_free(params);
 }
 
 static void event_who(IRC_SERVER_REC *server, const char *data)
 {
 	char *params, *nick, *channel, *user, *host, *stat, *realname, *hops;
+	char *serv;
 
 	g_return_if_fail(data != NULL);
 
 	params = event_get_params(data, 8, NULL, &channel, &user,
-				  &host, NULL, &nick, &stat, &realname);
+				  &host, &serv, &nick, &stat, &realname);
 
 	/* split hops/realname */
 	hops = realname;
@@ -116,7 +119,7 @@ static void event_who(IRC_SERVER_REC *server, const char *data)
 	if (realname > hops) realname[-1] = '\0';
 
 	printformat(server, NULL, MSGLEVEL_CRAP, IRCTXT_WHO,
-		    channel, nick, stat, hops, user, host, realname);
+		    channel, nick, stat, hops, user, host, realname, serv);
 
 	g_free(params);
 }
@@ -134,7 +137,10 @@ static void event_end_of_who(IRC_SERVER_REC *server, const char *data)
 
 static void event_ban_list(IRC_SERVER_REC *server, const char *data)
 {
-	char *params, *channel, *ban, *setby, *tims;
+	IRC_CHANNEL_REC *chanrec;
+	BAN_REC *banrec;
+	const char *channel;
+	char *params, *ban, *setby, *tims;
 	long secs;
 
 	g_return_if_fail(data != NULL);
@@ -144,16 +150,22 @@ static void event_ban_list(IRC_SERVER_REC *server, const char *data)
 	secs = *tims == '\0' ? 0 :
 		(long) (time(NULL) - atol(tims));
 
+	chanrec = irc_channel_find(server, channel);
+	banrec = chanrec == NULL ? NULL : banlist_find(chanrec->banlist, ban);
+
+	channel = get_visible_target(server, channel);
 	printformat(server, channel, MSGLEVEL_CRAP,
 		    *setby == '\0' ? IRCTXT_BANLIST : IRCTXT_BANLIST_LONG,
-		    0, channel, ban, setby, secs);
+		    banrec == NULL ? 0 : g_slist_index(chanrec->banlist, banrec)+1,
+		    channel, ban, setby, secs);
 
 	g_free(params);
 }
 
 static void event_eban_list(IRC_SERVER_REC *server, const char *data)
 {
-	char *params, *channel, *ban, *setby, *tims;
+	const char *channel;
+	char *params, *ban, *setby, *tims;
 	long secs;
 
 	g_return_if_fail(data != NULL);
@@ -163,6 +175,7 @@ static void event_eban_list(IRC_SERVER_REC *server, const char *data)
 	secs = *tims == '\0' ? 0 :
 		(long) (time(NULL) - atol(tims));
 
+	channel = get_visible_target(server, channel);
 	printformat(server, channel, MSGLEVEL_CRAP,
 		    *setby == '\0' ? IRCTXT_EBANLIST : IRCTXT_EBANLIST_LONG,
 		    channel, ban, setby, secs);
@@ -185,11 +198,13 @@ static void event_silence_list(IRC_SERVER_REC *server, const char *data)
 
 static void event_invite_list(IRC_SERVER_REC *server, const char *data)
 {
-	char *params, *channel, *invite;
+	const char *channel;
+	char *params, *invite;
 
 	g_return_if_fail(data != NULL);
 
 	params = event_get_params(data, 3, NULL, &channel, &invite);
+	channel = get_visible_target(server, channel);
 	printformat(server, channel, MSGLEVEL_CRAP,
 		    IRCTXT_INVITELIST, channel, invite);
 	g_free(params);
@@ -212,11 +227,13 @@ static void event_nick_in_use(IRC_SERVER_REC *server, const char *data)
 
 static void event_topic_get(IRC_SERVER_REC *server, const char *data)
 {
-	char *params, *channel, *topic;
+	const char *channel;
+	char *params, *topic;
 
 	g_return_if_fail(data != NULL);
 
 	params = event_get_params(data, 3, NULL, &channel, &topic);
+	channel = get_visible_target(server, channel);
 	printformat(server, channel, MSGLEVEL_CRAP,
 		    IRCTXT_TOPIC, channel, topic);
 	g_free(params);
@@ -224,29 +241,37 @@ static void event_topic_get(IRC_SERVER_REC *server, const char *data)
 
 static void event_topic_info(IRC_SERVER_REC *server, const char *data)
 {
-	char *params, *timestr, *channel, *topicby, *topictime;
+	const char *channel;
+	char *params, *timestr, *bynick, *byhost, *topictime;
 
 	g_return_if_fail(data != NULL);
 
 	params = event_get_params(data, 4, NULL, &channel,
-				  &topicby, &topictime);
+				  &bynick, &topictime);
 
         timestr = my_asctime((time_t) atol(topictime));
 
-	printformat(server, channel, MSGLEVEL_CRAP,
-		    IRCTXT_TOPIC_INFO, topicby, timestr);
+	byhost = strchr(bynick, '!');
+	if (byhost != NULL)
+		*byhost++ = '\0';
+
+	channel = get_visible_target(server, channel);
+	printformat(server, channel, MSGLEVEL_CRAP, IRCTXT_TOPIC_INFO,
+		    bynick, timestr, byhost == NULL ? "" : byhost);
 	g_free(timestr);
 	g_free(params);
 }
 
 static void event_channel_mode(IRC_SERVER_REC *server, const char *data)
 {
-	char *params, *channel, *mode;
+	const char *channel;
+	char *params, *mode;
 
 	g_return_if_fail(data != NULL);
 
 	params = event_get_params(data, 3 | PARAM_FLAG_GETREST,
 				  NULL, &channel, &mode);
+	channel = get_visible_target(server, channel);
 	printformat(server, channel, MSGLEVEL_CRAP,
 		    IRCTXT_CHANNEL_MODE, channel, g_strchomp(mode));
 	g_free(params);
@@ -254,20 +279,22 @@ static void event_channel_mode(IRC_SERVER_REC *server, const char *data)
 
 static void event_channel_created(IRC_SERVER_REC *server, const char *data)
 {
-	char *params, *channel, *createtime, *timestr;
+	const char *channel;
+	char *params, *createtime, *timestr;
 
 	g_return_if_fail(data != NULL);
 
 	params = event_get_params(data, 3, NULL, &channel, &createtime);
 
         timestr = my_asctime((time_t) atol(createtime));
+	channel = get_visible_target(server, channel);
 	printformat(server, channel, MSGLEVEL_CRAP,
 		    IRCTXT_CHANNEL_CREATED, channel, timestr);
 	g_free(timestr);
 	g_free(params);
 }
 
-static void event_away(IRC_SERVER_REC *server, const char *data)
+static void event_nowaway(IRC_SERVER_REC *server, const char *data)
 {
 	printformat(server, NULL, MSGLEVEL_CRAP, IRCTXT_AWAY);
 }
@@ -277,6 +304,29 @@ static void event_unaway(IRC_SERVER_REC *server, const char *data)
 	printformat(server, NULL, MSGLEVEL_CRAP, IRCTXT_UNAWAY);
 }
 
+static void event_away(IRC_SERVER_REC *server, const char *data)
+{
+	char *params, *nick, *awaymsg;
+
+	g_return_if_fail(data != NULL);
+
+	params = event_get_params(data, 3, NULL, &nick, &awaymsg);
+	if (!settings_get_bool("show_away_once") ||
+	    last_away_nick == NULL || g_strcasecmp(last_away_nick, nick) != 0 ||
+	    last_away_msg == NULL || g_strcasecmp(last_away_msg, awaymsg) != 0) {
+		/* don't show the same away message
+		   from the same nick all the time */
+		g_free_not_null(last_away_nick);
+		g_free_not_null(last_away_msg);
+		last_away_nick = g_strdup(nick);
+		last_away_msg = g_strdup(awaymsg);
+
+		printformat(server, nick, MSGLEVEL_CRAP,
+			    IRCTXT_NICK_AWAY, nick, awaymsg);
+	}
+	g_free(params);
+}
+
 static void event_userhost(IRC_SERVER_REC *server, const char *data)
 {
 	char *params, *hosts;
@@ -365,26 +415,36 @@ static void event_whois_oper(IRC_SERVER_REC *server, const char *data)
 	g_return_if_fail(data != NULL);
 
 	params = event_get_params(data, 3, NULL, &nick, &type);
+	/* type = "is an IRC Operator" */
 	if (strlen(type) > 5) {
 		type += 5;
 		if (*type == ' ') type++;
 	}
+	if (*type == '\0') {
+		/* shouldn't happen */
+		type = "IRC Operator";
+	}
 
 	printformat(server, nick, MSGLEVEL_CRAP,
-		    *type == '\0' ? IRCTXT_WHOIS_OPER :
-		    IRCTXT_WHOIS_OPER_TYPE, nick, type);
+		    IRCTXT_WHOIS_OPER, nick, type);
 	g_free(params);
 }
 
-static void event_whois_registered(IRC_SERVER_REC *server, const char *data)
+static void event_whois_registered(IRC_SERVER_REC *server, const char *data,
+				   const char *orignick, const char *addr)
 {
-	char *params, *nick;
+	char *params, *nick, *txt_identified;
 
 	g_return_if_fail(data != NULL);
 
-	params = event_get_params(data, 2, NULL, &nick);
-	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOIS_REGISTERED, nick);
+	params = event_get_params(data, 3, NULL, &nick, &txt_identified);
+	if (*txt_identified != '\0') {
+		printformat(server, nick, MSGLEVEL_CRAP,
+			    IRCTXT_WHOIS_REGISTERED, nick);
+	} else {
+		/* or /USERIP reply in undernet.. */
+		print_event_received(server, data, orignick, FALSE);
+	}
 	g_free(params);
 }
 
@@ -399,7 +459,8 @@ static void event_whois_help(IRC_SERVER_REC *server, const char *data)
 	g_free(params);
 }
 
-static void event_whois_modes(IRC_SERVER_REC *server, const char *data)
+static void event_whois_modes(IRC_SERVER_REC *server, const char *data,
+			      const char *orignick)
 {
 	char *params, *nick, *modes;
 
@@ -408,24 +469,96 @@ static void event_whois_modes(IRC_SERVER_REC *server, const char *data)
 	params = event_get_params(data, 6, NULL, &nick,
 				  NULL, NULL, NULL, &modes);
 
-	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOIS_MODES, nick, modes);
+	if (!ischannel(*nick)) {
+		printformat(server, nick, MSGLEVEL_CRAP,
+			    IRCTXT_WHOIS_MODES, nick, modes);
+	} else {
+		/* OPN's dancer uses for channel forwarding */
+		print_event_received(server, data, orignick, FALSE);
+	}
 	g_free(params);
 }
 
-static void event_whois_realhost(IRC_SERVER_REC *server, const char *data)
+static void event_whois_realhost(IRC_SERVER_REC *server, const char *data,
+				 const char *orignick, const char *addr)
 {
-	char *params, *nick, *str, *from;
+	char *params, *nick, *txt_real, *txt_hostname, *hostname;
 
 	g_return_if_fail(data != NULL);
 
-	params = event_get_params(data, 3, NULL, &nick, &str);
+        /* <yournick> real hostname <nick> <hostname> */
+	params = event_get_params(data, 5, NULL, &nick, &txt_real,
+				  &txt_hostname, &hostname);
+	if (strcmp(txt_real, "real") != 0 ||
+	    strcmp(txt_hostname, "hostname") != 0) {
+		/* <yournick> <nick> :... from <hostname> */
+                g_free(params);
+		params = event_get_params(data, 3, NULL, &nick, &hostname);
 
-	from = strstr(str, "from ");
-	if (from == NULL) from = str; else from += 5;
+		hostname = strstr(hostname, "from ");
+                if (hostname != NULL) hostname += 5;
+	}
 
+	if (hostname != NULL) {
+		printformat(server, nick, MSGLEVEL_CRAP,
+			    IRCTXT_WHOIS_REALHOST, nick, hostname, "");
+	} else {
+		/* OPN's dancer uses for end of /MAP */
+		print_event_received(server, data, orignick, FALSE);
+	}
+	g_free(params);
+}
+
+static void event_whois_usermode326(IRC_SERVER_REC *server, const char *data)
+{
+	char *params, *nick, *usermode;
+
+	g_return_if_fail(data != NULL);
+
+        /* <yournick> <nick> :has oper privs: <mode> */
+	params = event_get_params(data, 3, NULL, &nick, &usermode);
 	printformat(server, nick, MSGLEVEL_CRAP,
-		    IRCTXT_WHOIS_REALHOST, nick, from);
+		    IRCTXT_WHOIS_USERMODE, nick, usermode);
+        g_free(params);
+}
+
+static void event_whois_realhost327(IRC_SERVER_REC *server, const char *data,
+				    const char *orignick, const char *addr)
+{
+	char *params, *nick, *hostname, *ip, *text;
+
+	g_return_if_fail(data != NULL);
+
+	/* <yournick> <hostname> <ip> :Real hostname/IP */
+	params = event_get_params(data, 5, NULL, &nick, &hostname, &ip, &text);
+	if (*text != '\0') {
+		printformat(server, nick, MSGLEVEL_CRAP,
+			    IRCTXT_WHOIS_REALHOST, nick, hostname, ip);
+	} else {
+		print_event_received(server, data, orignick, FALSE);
+	}
+	g_free(params);
+}
+
+static void event_whois_usermode(IRC_SERVER_REC *server, const char *data,
+				 const char *orignick, const char *addr)
+{
+	char *params, *txt_usermodes, *nick, *usermode;
+
+	g_return_if_fail(data != NULL);
+
+	params = event_get_params(data, 4, NULL, &txt_usermodes,
+				  &nick, &usermode);
+
+	if (strcmp(txt_usermodes, "usermodes") == 0) {
+		/* <yournick> usermodes <nick> usermode */
+		printformat(server, nick, MSGLEVEL_CRAP,
+			    IRCTXT_WHOIS_USERMODE, nick, usermode);
+	} else {
+		/* some servers use this as motd too..
+		   and OPN's dancer for /MAP */
+		print_event_received(server, data, orignick, FALSE);
+	}
 	g_free(params);
 }
 
@@ -481,20 +614,8 @@ static void event_whois_away(IRC_SERVER_REC *server, const char *data)
 	g_return_if_fail(data != NULL);
 
 	params = event_get_params(data, 3, NULL, &nick, &awaymsg);
-	if (server->whois_coming || !settings_get_bool("show_away_once") ||
-	    last_away_nick == NULL || g_strcasecmp(last_away_nick, nick) != 0 ||
-	    last_away_msg == NULL || g_strcasecmp(last_away_msg, awaymsg) != 0) {
-		/* don't show the same away message
-		   from the same nick all the time */
-		g_free_not_null(last_away_nick);
-		g_free_not_null(last_away_msg);
-		last_away_nick = g_strdup(nick);
-		last_away_msg = g_strdup(awaymsg);
-
-		printformat(server, nick, MSGLEVEL_CRAP,
-			    server->whois_coming ? IRCTXT_WHOIS_AWAY :
-			    IRCTXT_NICK_AWAY, nick, awaymsg);
-	}
+	printformat(server, nick, MSGLEVEL_CRAP,
+		    IRCTXT_WHOIS_AWAY, nick, awaymsg);
 	g_free(params);
 }
 
@@ -514,16 +635,30 @@ static void event_end_of_whois(IRC_SERVER_REC *server, const char *data)
 
 static void event_chanserv_url(IRC_SERVER_REC *server, const char *data)
 {
-	char *params, *channel, *url;
+	const char *channel;
+	char *params, *url;
 
 	g_return_if_fail(data != NULL);
 
 	params = event_get_params(data, 3, NULL, &channel, &url);
+	channel = get_visible_target(server, channel);
 	printformat(server, channel, MSGLEVEL_CRAP,
 		    IRCTXT_CHANNEL_URL, channel, url);
 	g_free(params);
 }
 
+static void event_whois_auth(IRC_SERVER_REC *server, const char *data)
+{
+	char *params, *nick, *text;
+
+	g_return_if_fail(data != NULL);
+
+	params = event_get_params(data, 3, NULL, &nick, &text);
+	printformat(server, nick, MSGLEVEL_CRAP,
+		    IRCTXT_WHOIS_EXTRA, nick, text);
+	g_free(params);
+}
+
 static void event_end_of_whowas(IRC_SERVER_REC *server, const char *data)
 {
 	char *params, *nick;
@@ -538,7 +673,8 @@ static void event_end_of_whowas(IRC_SERVER_REC *server, const char *data)
 	g_free(params);
 }
 
-static void event_target_unavailable(IRC_SERVER_REC *server, const char *data)
+static void event_target_unavailable(IRC_SERVER_REC *server, const char *data,
+				     const char *nick, const char *addr)
 {
 	IRC_CHANNEL_REC *chanrec;
 	char *params, *target;
@@ -554,7 +690,7 @@ static void event_target_unavailable(IRC_SERVER_REC *server, const char *data)
 		chanrec = irc_channel_find(server, target);
 		if (chanrec != NULL && chanrec->joined) {
 			/* dalnet - can't change nick while being banned */
-			event_received(server, data);
+			print_event_received(server, data, nick, FALSE);
 		} else {
 			/* channel is unavailable. */
 			printformat(server, NULL, MSGLEVEL_CRAP,
@@ -660,55 +796,81 @@ static void event_unknown_mode(IRC_SERVER_REC *server, const char *data)
 	g_free(params);
 }
 
-static void event_not_chanop(IRC_SERVER_REC *server, const char *data)
+static void event_numeric(IRC_SERVER_REC *server, const char *data,
+			  const char *nick)
 {
-	char *params, *channel;
+	data = strchr(data, ' ');
+	if (data != NULL)
+                print_event_received(server, data+1, nick, FALSE);
+}
+
+static void print_event_received(IRC_SERVER_REC *server, const char *data,
+				 const char *nick, int target_param)
+{
+	char *target, *args, *ptr;
+	int format;
 
 	g_return_if_fail(data != NULL);
 
-	params = event_get_params(data, 2, NULL, &channel);
-	printformat(server, NULL, MSGLEVEL_CRAP, IRCTXT_NOT_CHANOP, channel);
-	g_free(params);
+        /* skip first param, it's always our nick */
+	ptr = strchr(data, ' ');
+	if (ptr == NULL)
+		return;
+	ptr++;
+
+	if (!target_param)
+		target = NULL;
+	else {
+                /* target parameter */
+		target = ptr;
+		ptr = strchr(target, ' ');
+		if (ptr == NULL)
+			return;
+
+                target = g_strndup(target, (int) (ptr-target));
+		ptr++;
+	}
+
+	/* param1 param2 ... :last parameter */
+	if (*ptr == ':') {
+                /* only one parameter */
+		args = g_strdup(ptr+1);
+	} else {
+		args = g_strdup(ptr);
+		ptr = strstr(args, " :");
+		if (ptr != NULL)
+			g_memmove(ptr+1, ptr+2, strlen(ptr+1));
+	}
+
+	format = nick == NULL || server->real_address == NULL ||
+		strcmp(nick, server->real_address) == 0 ?
+		IRCTXT_DEFAULT_EVENT : IRCTXT_DEFAULT_EVENT_SERVER;
+	printformat(server, target, MSGLEVEL_CRAP, format,
+		    nick, args, current_server_event);
+	g_free(args);
 }
 
-static void event_received(IRC_SERVER_REC *server, const char *data)
+static void event_received(IRC_SERVER_REC *server, const char *data,
+			   const char *nick)
 {
-	char *params, *args, *ptr;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 2 | PARAM_FLAG_GETREST, NULL, &args);
-	ptr = strstr(args, " :");
-	if (ptr != NULL)
-                g_memmove(ptr+1, ptr+2, strlen(ptr+1));
-	printtext(server, NULL, MSGLEVEL_CRAP, "%s", args);
-	g_free(params);
+        print_event_received(server, data, nick, FALSE);
 }
 
-static void event_target_received(IRC_SERVER_REC *server, const char *data)
+static void event_target_received(IRC_SERVER_REC *server, const char *data,
+				  const char *nick)
 {
-	char *params, *target, *args, *ptr;
-
-	g_return_if_fail(data != NULL);
-
-	params = event_get_params(data, 3 | PARAM_FLAG_GETREST,
-				  NULL, &target, &args);
-	ptr = strstr(args, " :");
-	if (ptr != NULL)
-                g_memmove(ptr+1, ptr+2, strlen(ptr+1));
-	printtext(server, target, MSGLEVEL_CRAP, "%s %s", target, args);
-	g_free(params);
+        print_event_received(server, data, nick, TRUE);
 }
 
-static void event_motd(IRC_SERVER_REC *server, const char *data)
+static void event_motd(IRC_SERVER_REC *server, const char *data,
+		       const char *nick, const char *addr)
 {
 	/* don't ignore motd anymore after 3 seconds of connection time -
 	   we might have called /MOTD */
-	if (settings_get_bool("skip_motd") &&
-	    time(NULL)-3 <= server->real_connect_time)
+	if (settings_get_bool("skip_motd") && !server->motd_got)
 		return;
 
-        event_received(server, data);
+        print_event_received(server, data, nick, FALSE);
 }
 
 static void sig_empty(void)
@@ -723,7 +885,7 @@ void fe_events_numeric_init(void)
 	signal_add("event 221", (SIGNAL_FUNC) event_user_mode);
 	signal_add("event 303", (SIGNAL_FUNC) event_ison);
 	signal_add("event 353", (SIGNAL_FUNC) event_names_list);
-	signal_add("event 366", (SIGNAL_FUNC) event_end_of_names);
+	signal_add_first("event 366", (SIGNAL_FUNC) event_end_of_names);
 	signal_add("event 352", (SIGNAL_FUNC) event_who);
 	signal_add("event 315", (SIGNAL_FUNC) event_end_of_who);
 	signal_add("event 271", (SIGNAL_FUNC) event_silence_list);
@@ -736,21 +898,28 @@ void fe_events_numeric_init(void)
 	signal_add("event 333", (SIGNAL_FUNC) event_topic_info);
 	signal_add("event 324", (SIGNAL_FUNC) event_channel_mode);
 	signal_add("event 329", (SIGNAL_FUNC) event_channel_created);
-	signal_add("event 306", (SIGNAL_FUNC) event_away);
+	signal_add("event 306", (SIGNAL_FUNC) event_nowaway);
 	signal_add("event 305", (SIGNAL_FUNC) event_unaway);
+	signal_add("event 301", (SIGNAL_FUNC) event_away);
 	signal_add("event 311", (SIGNAL_FUNC) event_whois);
-	signal_add("event 301", (SIGNAL_FUNC) event_whois_away);
+	signal_add("whois away", (SIGNAL_FUNC) event_whois_away);
+	signal_add("whowas away", (SIGNAL_FUNC) event_whois_away);
 	signal_add("event 312", (SIGNAL_FUNC) event_whois_server);
 	signal_add("event 313", (SIGNAL_FUNC) event_whois_oper);
 	signal_add("event 307", (SIGNAL_FUNC) event_whois_registered);
 	signal_add("event 310", (SIGNAL_FUNC) event_whois_help);
+	signal_add("event 326", (SIGNAL_FUNC) event_whois_usermode326);
+	signal_add("event 327", (SIGNAL_FUNC) event_whois_realhost327);
 	signal_add("event 379", (SIGNAL_FUNC) event_whois_modes);
 	signal_add("event 378", (SIGNAL_FUNC) event_whois_realhost);
+	signal_add("event 377", (SIGNAL_FUNC) event_whois_usermode);
 	signal_add("event 320", (SIGNAL_FUNC) event_whois_special);
+	signal_add("event 275", (SIGNAL_FUNC) event_whois_special);
 	signal_add("event 314", (SIGNAL_FUNC) event_whowas);
 	signal_add("event 317", (SIGNAL_FUNC) event_whois_idle);
 	signal_add("event 318", (SIGNAL_FUNC) event_end_of_whois);
 	signal_add("event 328", (SIGNAL_FUNC) event_chanserv_url);
+	signal_add("event 330", (SIGNAL_FUNC) event_whois_auth);
 	signal_add("event 369", (SIGNAL_FUNC) event_end_of_whowas);
 	signal_add("event 319", (SIGNAL_FUNC) event_whois_channels);
 	signal_add("event 302", (SIGNAL_FUNC) event_userhost);
@@ -767,24 +936,31 @@ void fe_events_numeric_init(void)
 	signal_add("event 474", (SIGNAL_FUNC) event_banned);
 	signal_add("event 475", (SIGNAL_FUNC) event_bad_channel_key);
 	signal_add("event 476", (SIGNAL_FUNC) event_bad_channel_mask);
-	signal_add("event 482", (SIGNAL_FUNC) event_not_chanop);
 	signal_add("event 375", (SIGNAL_FUNC) event_motd);
 	signal_add("event 376", (SIGNAL_FUNC) event_motd);
 	signal_add("event 372", (SIGNAL_FUNC) event_motd);
 	signal_add("event 422", (SIGNAL_FUNC) event_motd);
 
+        signal_add("default event numeric", (SIGNAL_FUNC) event_numeric);
+	signal_add("event 001", (SIGNAL_FUNC) event_received);
 	signal_add("event 004", (SIGNAL_FUNC) event_received);
 	signal_add("event 254", (SIGNAL_FUNC) event_received);
 	signal_add("event 364", (SIGNAL_FUNC) event_received);
 	signal_add("event 365", (SIGNAL_FUNC) event_received);
 	signal_add("event 381", (SIGNAL_FUNC) event_received);
+	signal_add("event 421", (SIGNAL_FUNC) event_received);
 	signal_add("event 432", (SIGNAL_FUNC) event_received);
+	signal_add("event 436", (SIGNAL_FUNC) event_received);
 	signal_add("event 438", (SIGNAL_FUNC) event_received);
 	signal_add("event 465", (SIGNAL_FUNC) event_received);
+	signal_add("event 439", (SIGNAL_FUNC) event_received);
+	signal_add("event 482", (SIGNAL_FUNC) event_received);
 
 	signal_add("event 368", (SIGNAL_FUNC) event_target_received);
 	signal_add("event 347", (SIGNAL_FUNC) event_target_received);
 	signal_add("event 349", (SIGNAL_FUNC) event_target_received);
+	signal_add("event 442", (SIGNAL_FUNC) event_target_received);
+	signal_add("event 477", (SIGNAL_FUNC) event_target_received);
 }
 
 void fe_events_numeric_deinit(void)
@@ -808,21 +984,28 @@ void fe_events_numeric_deinit(void)
 	signal_remove("event 333", (SIGNAL_FUNC) event_topic_info);
 	signal_remove("event 324", (SIGNAL_FUNC) event_channel_mode);
 	signal_remove("event 329", (SIGNAL_FUNC) event_channel_created);
-	signal_remove("event 306", (SIGNAL_FUNC) event_away);
+	signal_remove("event 306", (SIGNAL_FUNC) event_nowaway);
 	signal_remove("event 305", (SIGNAL_FUNC) event_unaway);
+	signal_remove("event 301", (SIGNAL_FUNC) event_away);
 	signal_remove("event 311", (SIGNAL_FUNC) event_whois);
-	signal_remove("event 301", (SIGNAL_FUNC) event_whois_away);
+	signal_remove("whois away", (SIGNAL_FUNC) event_whois_away);
+	signal_remove("whowas away", (SIGNAL_FUNC) event_whois_away);
 	signal_remove("event 312", (SIGNAL_FUNC) event_whois_server);
 	signal_remove("event 313", (SIGNAL_FUNC) event_whois_oper);
 	signal_remove("event 307", (SIGNAL_FUNC) event_whois_registered);
 	signal_remove("event 310", (SIGNAL_FUNC) event_whois_help);
+	signal_remove("event 326", (SIGNAL_FUNC) event_whois_usermode326);
+	signal_remove("event 327", (SIGNAL_FUNC) event_whois_realhost327);
 	signal_remove("event 379", (SIGNAL_FUNC) event_whois_modes);
 	signal_remove("event 378", (SIGNAL_FUNC) event_whois_realhost);
+	signal_remove("event 377", (SIGNAL_FUNC) event_whois_usermode);
 	signal_remove("event 320", (SIGNAL_FUNC) event_whois_special);
+	signal_remove("event 275", (SIGNAL_FUNC) event_whois_special);
 	signal_remove("event 314", (SIGNAL_FUNC) event_whowas);
 	signal_remove("event 317", (SIGNAL_FUNC) event_whois_idle);
 	signal_remove("event 318", (SIGNAL_FUNC) event_end_of_whois);
 	signal_remove("event 328", (SIGNAL_FUNC) event_chanserv_url);
+	signal_remove("event 330", (SIGNAL_FUNC) event_whois_auth);
 	signal_remove("event 369", (SIGNAL_FUNC) event_end_of_whowas);
 	signal_remove("event 319", (SIGNAL_FUNC) event_whois_channels);
 	signal_remove("event 302", (SIGNAL_FUNC) event_userhost);
@@ -839,22 +1022,29 @@ void fe_events_numeric_deinit(void)
 	signal_remove("event 474", (SIGNAL_FUNC) event_banned);
 	signal_remove("event 475", (SIGNAL_FUNC) event_bad_channel_key);
 	signal_remove("event 476", (SIGNAL_FUNC) event_bad_channel_mask);
-	signal_remove("event 482", (SIGNAL_FUNC) event_not_chanop);
 	signal_remove("event 375", (SIGNAL_FUNC) event_motd);
 	signal_remove("event 376", (SIGNAL_FUNC) event_motd);
 	signal_remove("event 372", (SIGNAL_FUNC) event_motd);
 	signal_remove("event 422", (SIGNAL_FUNC) event_motd);
 
+        signal_remove("default event numeric", (SIGNAL_FUNC) event_numeric);
+	signal_remove("event 001", (SIGNAL_FUNC) event_received);
 	signal_remove("event 004", (SIGNAL_FUNC) event_received);
 	signal_remove("event 254", (SIGNAL_FUNC) event_received);
 	signal_remove("event 364", (SIGNAL_FUNC) event_received);
 	signal_remove("event 365", (SIGNAL_FUNC) event_received);
 	signal_remove("event 381", (SIGNAL_FUNC) event_received);
+	signal_remove("event 421", (SIGNAL_FUNC) event_received);
 	signal_remove("event 432", (SIGNAL_FUNC) event_received);
+	signal_remove("event 436", (SIGNAL_FUNC) event_received);
 	signal_remove("event 438", (SIGNAL_FUNC) event_received);
 	signal_remove("event 465", (SIGNAL_FUNC) event_received);
+	signal_remove("event 439", (SIGNAL_FUNC) event_received);
+	signal_remove("event 482", (SIGNAL_FUNC) event_received);
 
 	signal_remove("event 368", (SIGNAL_FUNC) event_target_received);
 	signal_remove("event 347", (SIGNAL_FUNC) event_target_received);
 	signal_remove("event 349", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 442", (SIGNAL_FUNC) event_target_received);
+	signal_remove("event 477", (SIGNAL_FUNC) event_target_received);
 }
