@@ -23,7 +23,6 @@
 #include "network.h"
 #include "signals.h"
 
-#include "irc.h"
 #include "modes.h"
 #include "irc-servers.h"
 
@@ -40,7 +39,9 @@ static void sig_server_connect_copy(SERVER_CONNECT_REC **dest,
 
 	rec = g_new0(IRC_SERVER_CONNECT_REC, 1);
 	rec->chat_type = IRC_PROTOCOL;
+	rec->max_cmds_at_once = src->max_cmds_at_once;
 	rec->cmd_queue_speed = src->cmd_queue_speed;
+        rec->max_query_chans = src->max_query_chans;
 	rec->max_kicks = src->max_kicks;
 	rec->max_modes = src->max_modes;
 	rec->max_msgs = src->max_msgs;
@@ -51,40 +52,15 @@ static void sig_server_connect_copy(SERVER_CONNECT_REC **dest,
 static void sig_server_reconnect_save_status(IRC_SERVER_CONNECT_REC *conn,
 					     IRC_SERVER_REC *server)
 {
-	if (!IS_IRC_SERVER_CONNECT(conn) || !IS_IRC_SERVER(server))
+	if (!IS_IRC_SERVER_CONNECT(conn) || !IS_IRC_SERVER(server) ||
+	    !server->connected)
 		return;
 
 	g_free_not_null(conn->channels);
 	conn->channels = irc_server_get_channels(server);
 
 	g_free_not_null(conn->usermode);
-	conn->usermode = g_strdup(server->usermode);
-}
-
-static int sig_set_user_mode(IRC_SERVER_REC *server)
-{
-	const char *mode;
-	char *newmode;
-
-	if (g_slist_find(servers, server) == NULL)
-		return 0; /* got disconnected */
-
-	mode = server->connrec->usermode;
-	if (mode == NULL) return 0;
-
-	newmode = server->usermode == NULL ? NULL :
-		modes_join(server->usermode, mode, FALSE);
-
-	if (server->usermode == NULL) {
-		/* server didn't set user mode, just set the new one */
-		irc_send_cmdv(server, "MODE %s %s", server->nick, mode);
-	} else {
-		if (strcmp(newmode, server->usermode) != 0)
-			irc_send_cmdv(server, "MODE %s -%s+%s", server->nick, server->usermode, mode);
-	}
-
-	g_free_not_null(newmode);
-	return 0;
+	conn->usermode = g_strdup(server->wanted_usermode);
 }
 
 static void sig_connected(IRC_SERVER_REC *server)
@@ -93,29 +69,35 @@ static void sig_connected(IRC_SERVER_REC *server)
 		return;
 
 	if (server->connrec->away_reason != NULL)
-		signal_emit("command away", 2, server->connrec->away_reason, server, NULL);
-	if (server->connrec->usermode != NULL) {
-		/* wait a second and then send the user mode */
-		g_timeout_add(1000, (GSourceFunc) sig_set_user_mode, server);
-	}
+		irc_send_cmdv(server, "AWAY :%s", server->connrec->away_reason);
 }
 
-static void event_kill(IRC_SERVER_REC *server, const char *data,
-		       const char *nick, const char *addr)
+static void event_nick_collision(IRC_SERVER_REC *server, const char *data)
 {
 	time_t new_connect;
 
-	if (!IS_IRC_SERVER(server)/* || addr != NULL*/)
+	if (!IS_IRC_SERVER(server))
 		return;
 
-	/* after server kills we want to connect back immediately - it was
-	   probably a nick collision. but no matter how hard they kill us,
+	/* after server kills us because of nick collision, we want to
+	   connect back immediately. but no matter how hard they kill us,
 	   don't connect to the server more than once in every 10 seconds. */
 
 	new_connect = server->connect_time+10 -
 		settings_get_int("server_reconnect_time");
 	if (server->connect_time > new_connect)
 		server->connect_time = new_connect;
+
+        server->nick_collision = TRUE;
+}
+
+static void event_kill(IRC_SERVER_REC *server, const char *data,
+		       const char *nick, const char *addr)
+{
+	if (addr != NULL && !server->nick_collision) {
+		/* don't reconnect if we were killed by an oper (not server) */
+		server->no_reconnect = TRUE;
+	}
 }
 
 void irc_servers_reconnect_init(void)
@@ -123,6 +105,7 @@ void irc_servers_reconnect_init(void)
 	signal_add("server connect copy", (SIGNAL_FUNC) sig_server_connect_copy);
 	signal_add("server reconnect save status", (SIGNAL_FUNC) sig_server_reconnect_save_status);
 	signal_add("event connected", (SIGNAL_FUNC) sig_connected);
+	signal_add("event 436", (SIGNAL_FUNC) event_nick_collision);
 	signal_add("event kill", (SIGNAL_FUNC) event_kill);
 }
 
@@ -131,5 +114,6 @@ void irc_servers_reconnect_deinit(void)
 	signal_remove("server connect copy", (SIGNAL_FUNC) sig_server_connect_copy);
 	signal_remove("server reconnect save status", (SIGNAL_FUNC) sig_server_reconnect_save_status);
 	signal_remove("event connected", (SIGNAL_FUNC) sig_connected);
+	signal_remove("event 436", (SIGNAL_FUNC) event_nick_collision);
 	signal_remove("event kill", (SIGNAL_FUNC) event_kill);
 }
