@@ -32,9 +32,10 @@
 
 /* Change nick's mode in channel */
 static void nick_mode_change(IRC_CHANNEL_REC *channel, const char *nick,
-			     const char mode, int type)
+			     const char mode, int type, const char *setby)
 {
 	NICK_REC *nickrec;
+	char modestr[2], typestr[2];
 
 	g_return_if_fail(IS_IRC_CHANNEL(channel));
 	g_return_if_fail(nick != NULL);
@@ -46,7 +47,10 @@ static void nick_mode_change(IRC_CHANNEL_REC *channel, const char *nick,
 	if (mode == '+') nickrec->voice = type == '+';
 	if (mode == '%') nickrec->halfop = type == '+';
 
-	signal_emit("nick mode changed", 2, channel, nickrec);
+	modestr[0] = mode; modestr[1] = '\0';
+	typestr[0] = type; typestr[1] = '\0';
+	signal_emit("nick mode changed", 5,
+		    channel, nickrec, setby, modestr, typestr);
 }
 
 static int mode_is_set(const char *str, char mode)
@@ -198,7 +202,7 @@ int channel_mode_is_set(IRC_CHANNEL_REC *channel, char mode)
 
 /* Parse channel mode string */
 void parse_channel_modes(IRC_CHANNEL_REC *channel, const char *setby,
-			 const char *mode)
+			 const char *mode, int update_key)
 {
         GString *newmode;
 	char *dup, *modestr, *arg, *curmode, type;
@@ -236,13 +240,13 @@ void parse_channel_modes(IRC_CHANNEL_REC *channel, const char *setby,
 		case 'O': /* channel owner in !channels */
 			if (g_strcasecmp(channel->server->nick, arg) == 0)
 				channel->chanop = type == '+';
-			nick_mode_change(channel, arg, '@', type);
+			nick_mode_change(channel, arg, '@', type, setby);
 			break;
 		case 'h':
-			nick_mode_change(channel, arg, '%', type);
+			nick_mode_change(channel, arg, '%', type, setby);
 			break;
 		case 'v':
-			nick_mode_change(channel, arg, '+', type);
+			nick_mode_change(channel, arg, '+', type, setby);
 			break;
 
 		case 'l':
@@ -250,7 +254,8 @@ void parse_channel_modes(IRC_CHANNEL_REC *channel, const char *setby,
 			channel->limit = type == '-' ? 0 : atoi(arg);
 			break;
 		case 'k':
-			if (*arg == '\0' && type == '+') {
+			if ((*arg == '\0' && type == '+') ||
+			    (channel->key != NULL && !update_key)) {
 				arg = channel->key != NULL ? channel->key :
 					"???";
 			}
@@ -265,6 +270,7 @@ void parse_channel_modes(IRC_CHANNEL_REC *channel, const char *setby,
 		case 'e':
 		case 'I':
 		case 'q':
+		case 'd':
 			/* Don't set it as channel mode */
 			break;
 
@@ -290,7 +296,7 @@ void parse_channel_modes(IRC_CHANNEL_REC *channel, const char *setby,
 		g_free(channel->mode);
 		channel->mode = g_strdup(newmode->str);
 
-		signal_emit("channel mode changed", 1, channel);
+		signal_emit("channel mode changed", 2, channel, setby);
 	}
 
 	g_string_free(newmode, TRUE);
@@ -381,7 +387,7 @@ static void event_mode(IRC_SERVER_REC *server, const char *data,
 		/* channel mode change */
 		chanrec = irc_channel_find(server, channel);
 		if (chanrec != NULL)
-			parse_channel_modes(chanrec, nick, mode);
+			parse_channel_modes(chanrec, nick, mode, TRUE);
 	}
 
 	g_free(params);
@@ -445,15 +451,6 @@ void channel_set_singlemode(IRC_CHANNEL_REC *channel, const char *nicks,
 	g_return_if_fail(nicks != NULL && mode != NULL);
 	if (*nicks == '\0') return;
 
-	if (!channel->chanop) {
-                /* not op - can we do anything? */
-		if (channel->ownnick == NULL || !channel->ownnick->halfop)
-			return; /* not even halfop, abort */
-
-		if (mode[0] != '\0' && mode[1] == 'o')
-                        return; /* halfops can't op/deop */
-	}
-
 	num = modepos = 0;
 	str = g_string_new(NULL);
 
@@ -489,8 +486,9 @@ void channel_set_singlemode(IRC_CHANNEL_REC *channel, const char *nicks,
 void channel_set_mode(IRC_SERVER_REC *server, const char *channel,
 		      const char *mode)
 {
-	char *modestr, *curmode, *orig, type, prevtype;
+	IRC_CHANNEL_REC *chanrec;
 	GString *tmode, *targs;
+	char *modestr, *curmode, *orig, type, prevtype;
 	int count;
 
 	g_return_if_fail(IS_IRC_SERVER(server));
@@ -500,11 +498,23 @@ void channel_set_mode(IRC_SERVER_REC *server, const char *channel,
 	targs = g_string_new(NULL);
 	count = 0;
 
+	chanrec = irc_channel_find(server, channel);
+	if (chanrec != NULL)
+		channel = chanrec->name;
+
 	orig = modestr = g_strdup(mode);
 
         type = '+'; prevtype = '\0';
 	curmode = cmd_get_param(&modestr);
-	for (; *curmode != '\0'; curmode++) {
+	for (;; curmode++) {
+		if (*curmode == '\0') {
+			/* support for +o nick +o nick2 */
+			curmode = cmd_get_param(&modestr);
+			if (*curmode == '\0')
+				break;
+			continue;
+		}
+
 		if (*curmode == '+' || *curmode == '-') {
 			type = *curmode;
 			continue;
@@ -710,6 +720,7 @@ static void cmd_devoice(const char *data, IRC_SERVER_REC *server,
 static void cmd_mode(const char *data, IRC_SERVER_REC *server,
 		     IRC_CHANNEL_REC *channel)
 {
+	IRC_CHANNEL_REC *chanrec;
 	char *target, *mode;
 	void *free_arg;
 
@@ -732,9 +743,13 @@ static void cmd_mode(const char *data, IRC_SERVER_REC *server,
 	}
 	if (*target == '\0') cmd_param_error(CMDERR_NOT_ENOUGH_PARAMS);
 
-	if (*mode == '\0')
+	if (*mode == '\0') {
+		chanrec = irc_channel_find(server, target);
+		if (chanrec != NULL)
+			target = chanrec->name;
+
 		irc_send_cmdv(server, "MODE %s", target);
-	else if (ischannel(*target))
+	} else if (ischannel(*target))
 		channel_set_mode(server, target, mode);
 	else {
 		if (g_strcasecmp(target, server->nick) == 0) {
