@@ -28,10 +28,9 @@
 #include "misc.h"
 #include "settings.h"
 
-#include "masks.h"
-#include "irc.h"
-#include "servers-setup.h"
+#include "irc-servers.h"
 #include "irc-queries.h"
+#include "masks.h"
 
 #include "dcc-chat.h"
 
@@ -174,39 +173,45 @@ CHAT_DCC_REC *item_get_dcc(WI_ITEM_REC *item)
 }
 
 /* Send text to DCC chat */
-static void cmd_msg(const char *data)
+static void cmd_msg(const char *data, SERVER_REC *server, WI_ITEM_REC *item)
 {
 	CHAT_DCC_REC *dcc;
+        GHashTable *optlist;
 	char *text, *target;
 	void *free_arg;
 
 	g_return_if_fail(data != NULL);
 
-	if (*data != '=') {
-		/* handle only DCC messages */
-		return;
-	}
-
-	if (!cmd_get_params(data, &free_arg, 2 | PARAM_FLAG_GETREST,
-			    &target, &text))
+	if (!cmd_get_params(data, &free_arg, 2 | PARAM_FLAG_UNKNOWN_OPTIONS |
+			    PARAM_FLAG_OPTIONS | PARAM_FLAG_GETREST, "msg",
+			    &optlist, &target, &text))
 		return;
 
-	dcc = dcc_chat_find_id(target+1);
+	/* handle only DCC messages */
+	if (strcmp(target, "*") == 0)
+		dcc = item_get_dcc(item);
+	else if (*target == '=')
+		dcc = dcc_chat_find_id(target+1);
+	else
+		dcc = NULL;
+
 	if (dcc != NULL && dcc->sendbuf != NULL)
 		dcc_chat_send(dcc, text);
 
+	if (dcc != NULL || *target == '=')
+		signal_stop();
+
 	cmd_params_free(free_arg);
-	signal_stop();
 }
 
-static void cmd_me(const char *data, IRC_SERVER_REC *server, QUERY_REC *item)
+static void cmd_me(const char *data, SERVER_REC *server, WI_ITEM_REC *item)
 {
 	CHAT_DCC_REC *dcc;
 	char *str;
 
 	g_return_if_fail(data != NULL);
 
-	dcc = item_get_dcc((WI_ITEM_REC *) item);
+	dcc = item_get_dcc(item);
 	if (dcc == NULL) return;
 
 	str = g_strconcat("ACTION ", data, NULL);
@@ -216,7 +221,7 @@ static void cmd_me(const char *data, IRC_SERVER_REC *server, QUERY_REC *item)
 	signal_stop();
 }
 
-static void cmd_action(const char *data, IRC_SERVER_REC *server)
+static void cmd_action(const char *data, SERVER_REC *server)
 {
 	CHAT_DCC_REC *dcc;
 	char *target, *text, *str;
@@ -246,15 +251,13 @@ static void cmd_action(const char *data, IRC_SERVER_REC *server)
 	signal_stop();
 }
 
-static void cmd_ctcp(const char *data, IRC_SERVER_REC *server)
+static void cmd_ctcp(const char *data, SERVER_REC *server)
 {
 	CHAT_DCC_REC *dcc;
 	char *target, *ctcpcmd, *ctcpdata, *str;
 	void *free_arg;
 
 	g_return_if_fail(data != NULL);
-	if (server == NULL || !server->connected)
-		cmd_return_error(CMDERR_NOT_CONNECTED);
 
 	if (!cmd_get_params(data, &free_arg, 3 | PARAM_FLAG_GETREST,
 			    &target, &ctcpcmd, &ctcpdata))
@@ -323,8 +326,9 @@ static void dcc_chat_listen(CHAT_DCC_REC *dcc)
 
 	/* TODO: add paranoia check - see dcc-files.c */
 
-	g_source_remove(dcc->tagconn);
 	net_disconnect(dcc->handle);
+	g_source_remove(dcc->tagconn);
+	dcc->tagconn = -1;
 
 	dcc->starttime = time(NULL);
 	dcc->handle = handle;
@@ -352,6 +356,8 @@ static void sig_chat_connected(CHAT_DCC_REC *dcc)
 
 	/* connect ok. */
 	g_source_remove(dcc->tagconn);
+	dcc->tagconn = -1;
+
 	dcc->starttime = time(NULL);
 	dcc->sendbuf = net_sendbuffer_create(dcc->handle, 0);
 	dcc->tagread = g_input_add(dcc->handle, G_INPUT_READ,
@@ -362,8 +368,6 @@ static void sig_chat_connected(CHAT_DCC_REC *dcc)
 
 static void dcc_chat_connect(CHAT_DCC_REC *dcc)
 {
-	IPADDR *own_ip;
-
 	g_return_if_fail(IS_DCC_CHAT(dcc));
 
 	if (dcc->addrstr[0] == '\0' ||
@@ -372,8 +376,7 @@ static void dcc_chat_connect(CHAT_DCC_REC *dcc)
 		return;
 	}
 
-	own_ip = IPADDR_IS_V6(&dcc->addr) ? source_host_ip6 : source_host_ip4;
-	dcc->handle = net_connect_ip(&dcc->addr, dcc->port, own_ip);
+	dcc->handle = dcc_connect_ip(&dcc->addr, dcc->port);
 	if (dcc->handle != NULL) {
 		dcc->tagconn = g_input_add(dcc->handle,
 					   G_INPUT_WRITE | G_INPUT_READ,
@@ -425,7 +428,7 @@ static void cmd_dcc_chat(const char *data, IRC_SERVER_REC *server)
 	}
 
 	/* start listening  */
-	if (server == NULL || !server->connected)
+	if (!IS_IRC_SERVER(server) || !server->connected)
 		cmd_param_error(CMDERR_NOT_CONNECTED);
 
 	handle = dcc_listen(net_sendbuffer_handle(server->handle),
@@ -439,6 +442,8 @@ static void cmd_dcc_chat(const char *data, IRC_SERVER_REC *server)
 				   (GInputFunction) dcc_chat_listen, dcc);
 
 	/* send the chat request */
+	signal_emit("dcc request send", 1, dcc);
+
 	dcc_ip2str(&own_ip, host);
 	irc_send_cmdv(server, "PRIVMSG %s :\001DCC CHAT CHAT %s %d\001",
 		      nick, host, port);
@@ -447,7 +452,7 @@ static void cmd_dcc_chat(const char *data, IRC_SERVER_REC *server)
 }
 
 /* SYNTAX: MIRCDCC ON|OFF */
-static void cmd_mircdcc(const char *data, IRC_SERVER_REC *server,
+static void cmd_mircdcc(const char *data, SERVER_REC *server,
 			QUERY_REC *item)
 {
 	CHAT_DCC_REC *dcc;
@@ -457,13 +462,13 @@ static void cmd_mircdcc(const char *data, IRC_SERVER_REC *server,
 	dcc = item_get_dcc((WI_ITEM_REC *) item);
 	if (dcc == NULL) return;
 
-	dcc->mirc_ctcp = toupper(*data) != 'N' &&
+	dcc->mirc_ctcp = i_toupper(*data) != 'N' &&
 		g_strncasecmp(data, "OF", 2) != 0;
 }
 
 /* DCC CLOSE CHAT <nick> - check only from chat_ids in open DCC chats,
    the default handler will check from DCC chat requests */
-static void cmd_dcc_close(char *data, IRC_SERVER_REC *server)
+static void cmd_dcc_close(char *data, SERVER_REC *server)
 {
 	GSList *tmp, *next;
 	char *nick;
@@ -486,8 +491,8 @@ static void cmd_dcc_close(char *data, IRC_SERVER_REC *server)
 		if (IS_DCC_CHAT(dcc) && dcc->id != NULL &&
 		    g_strcasecmp(dcc->id, nick) == 0) {
 			found = TRUE;
-			if (!dcc_is_connected(dcc))
-				dcc_reject(DCC(dcc), server);
+			if (!dcc_is_connected(dcc) && IS_IRC_SERVER(server))
+				dcc_reject(DCC(dcc), IRC_SERVER(server));
 			else {
 				/* don't send DCC REJECT after DCC chat
 				   is already open */
@@ -501,7 +506,7 @@ static void cmd_dcc_close(char *data, IRC_SERVER_REC *server)
 	cmd_params_free(free_arg);
 }
 
-static void cmd_whois(const char *data, IRC_SERVER_REC *server,
+static void cmd_whois(const char *data, SERVER_REC *server,
 		      WI_ITEM_REC *item)
 {
 	CHAT_DCC_REC *dcc;
@@ -520,7 +525,7 @@ static void cmd_whois(const char *data, IRC_SERVER_REC *server,
 }
 
 #define DCC_AUTOACCEPT_PORT(dcc) \
-	((dcc)->port >= 1024 || settings_get_bool("dcc_autoaccept_lowport"))
+	((dcc)->port >= 1024 || settings_get_bool("dcc_autoaccept_lowports"))
 
 #define DCC_CHAT_AUTOACCEPT(dcc, server, nick, addr) \
 	(DCC_AUTOACCEPT_PORT(dcc) && \
@@ -578,7 +583,7 @@ static void ctcp_msg_dcc_chat(IRC_SERVER_REC *server, const char *data,
 /* DCC CHAT: text received */
 static void dcc_chat_msg(CHAT_DCC_REC *dcc, const char *msg)
 {
-	char *cmd, *ptr;
+	char *event, *cmd, *ptr;
 	int reply;
 
 	g_return_if_fail(IS_DCC_CHAT(dcc));
@@ -604,23 +609,29 @@ static void dcc_chat_msg(CHAT_DCC_REC *dcc, const char *msg)
 		return;
 
 	/* get ctcp command, remove \001 chars */
-	cmd = g_strconcat(reply ? "dcc reply " : "dcc ctcp ", msg+1, NULL);
-	if (cmd[strlen(cmd)-1] == 1) cmd[strlen(cmd)-1] = '\0';
+	event = g_strconcat(reply ? "dcc reply " : "dcc ctcp ", msg+1, NULL);
+	if (event[strlen(event)-1] == 1) event[strlen(event)-1] = '\0';
 
-	ptr = strchr(cmd+(reply ? 10 : 9), ' ');
+        cmd = event + (reply ? 10 : 9);
+	ptr = strchr(cmd, ' ');
 	if (ptr != NULL) *ptr++ = '\0'; else ptr = "";
 
-	g_strdown(cmd+9);
-	if (!signal_emit(cmd, 2, ptr, dcc)) {
+	cmd = g_strdup(cmd);
+	g_strup(cmd);
+
+	g_strdown(event+9);
+	if (!signal_emit(event, 2, dcc, ptr)) {
 		signal_emit(reply ? "default dcc reply" :
-			    "default dcc ctcp", 2, msg, dcc);
+			    "default dcc ctcp", 3, dcc, cmd, ptr);
 	}
-	g_free(cmd);
+
+        g_free(cmd);
+	g_free(event);
 
 	signal_stop();
 }
 
-static void dcc_ctcp_redirect(const char *msg, CHAT_DCC_REC *dcc)
+static void dcc_ctcp_redirect(CHAT_DCC_REC *dcc, const char *msg)
 {
 	g_return_if_fail(msg != NULL);
 	g_return_if_fail(IS_DCC_CHAT(dcc));
@@ -629,7 +640,7 @@ static void dcc_ctcp_redirect(const char *msg, CHAT_DCC_REC *dcc)
 		    dcc->nick, "dcc", dcc->mynick, dcc);
 }
 
-static void dcc_ctcp_reply_redirect(const char *msg, CHAT_DCC_REC *dcc)
+static void dcc_ctcp_reply_redirect(CHAT_DCC_REC *dcc, const char *msg)
 {
 	g_return_if_fail(msg != NULL);
 	g_return_if_fail(IS_DCC_CHAT(dcc));
