@@ -1,7 +1,7 @@
 /*
- listen.c : sample plugin for irssi
+ listen.c : irc proxy
 
-    Copyright (C) 1999 Timo Sirainen
+    Copyright (C) 1999-2001 Timo Sirainen
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,77 +28,79 @@
 #include "irc.h"
 #include "irc-channels.h"
 
-#include "fe-common/core/printtext.h"
+#include "fe-common/core/printtext.h" /* FIXME: evil. need to do fe-proxy */
 
 GSList *proxy_listens;
 GSList *proxy_clients;
 
+static CLIENT_REC *ctcp_client;
 static GString *next_line;
+static int ignore_next;
 
 static void remove_client(CLIENT_REC *rec)
 {
 	g_return_if_fail(rec != NULL);
 
 	proxy_clients = g_slist_remove(proxy_clients, rec);
+	rec->listen->clients = g_slist_remove(rec->listen->clients, rec);
+
+	if (ctcp_client == rec)
+                ctcp_client = NULL;
+
+	signal_emit("proxy client disconnected", 1, rec);
+	printtext(NULL, NULL, MSGLEVEL_CLIENTNOTICE,
+		  "Proxy: Client disconnected from %s", rec->host);
 
 	g_free(rec->proxy_address);
 	net_disconnect(rec->handle);
 	g_source_remove(rec->tag);
 	line_split_free(rec->buffer);
 	g_free_not_null(rec->nick);
+	g_free_not_null(rec->host);
 	g_free(rec);
 }
 
-static void proxy_redirect_event(CLIENT_REC *client,
-				 const char *args, int last, ...)
+static void proxy_redirect_event(CLIENT_REC *client, const char *command,
+				 int count, const char *arg, int remote)
 {
-	va_list vargs;
-	GString *str;
-	char *event;
-	int argpos, group;
+	char *str;
 
 	g_return_if_fail(client != NULL);
 
-	va_start(vargs, last);
-
-	str = g_string_new(NULL);
-	group = 0;
-	while ((event = va_arg(vargs, char *)) != NULL) {
-		argpos = va_arg(vargs, int);
-		g_string_sprintf(str, "proxy %p", client->handle);
-		group = server_redirect_single_event(SERVER(client->server), args, last > 0,
-						     group, event, str->str, argpos);
-		last--;
-	}
-	g_string_free(str, TRUE);
-
-	va_end(vargs);
+	str = g_strdup_printf("proxy %p", client);
+	server_redirect_event(client->server, command, count,
+			      arg, remote, NULL, "", str, NULL);
+	g_free(str);
 }
 
 static void grab_who(CLIENT_REC *client, const char *channel)
 {
-	char *chlist, *chanevent;
+	GString *arg;
 	char **list, **tmp;
+	int count;
 
 	/* /WHO a,b,c may respond with either one "a,b,c End of WHO" message
 	   or three different "a End of WHO", "b End of WHO", .. messages */
-	chlist = g_strdup(channel);
 	list = g_strsplit(channel, ",", -1);
 
-	for (tmp = list; *tmp != NULL; tmp++) {
+	arg = g_string_new(channel);
+
+	for (tmp = list, count = 0; *tmp != NULL; tmp++, count++) {
 		if (strcmp(*tmp, "0") == 0) {
 			/* /who 0 displays everyone */
 			**tmp = '*';
 		}
 
-		chanevent = g_strdup_printf("%s %s", chlist, *tmp);
-		proxy_redirect_event(client, chanevent, 2,
-				     "event 401", 1, "event 315", 1,
-				     "event 352", -1, NULL);
-		g_free(chanevent);
+		g_string_append_c(arg, ' ');
+		g_string_append(arg, *tmp);
 	}
+
+	proxy_redirect_event(client, "who",
+			     client->server->one_endofwho ? 1 : count,
+			     arg->str, -1);
+
 	g_strfreev(list);
-	g_free(chlist);
+	g_string_free(arg, TRUE);
 }
 
 static void handle_client_connect_cmd(CLIENT_REC *client,
@@ -114,26 +116,29 @@ static void handle_client_connect_cmd(CLIENT_REC *client,
 		else {
 			/* wrong password! */
 			remove_client(client);
+                        return;
 		}
 	} else if (strcmp(cmd, "NICK") == 0) {
 		g_free_not_null(client->nick);
 		client->nick = g_strdup(args);
 	} else if (strcmp(cmd, "USER") == 0) {
-		if (client->nick == NULL ||
-		    (*password != '\0' && !client->pass_sent)) {
-			/* stupid client didn't send us NICK/PASS, kill it */
+		client->user_sent = TRUE;
+	}
+
+	if (client->nick != NULL && client->user_sent) {
+		if (*password != '\0' && !client->pass_sent) {
+			/* client didn't send us PASS, kill it */
 			remove_client(client);
 		} else {
 			client->connected = TRUE;
-			plugin_proxy_dump_data(client);
+			proxy_dump_data(client);
 		}
 	}
 }
 
-static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args)
+static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args,
+			      const char *data)
 {
-	GIOChannel *server_handle;
-
 	if (!client->connected) {
 		handle_client_connect_cmd(client, cmd, args);
 		return;
@@ -143,39 +148,47 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args)
 		remove_client(client);
 		return;
 	}
+
 	if (strcmp(cmd, "PING") == 0) {
-		char *server = strchr(args, ':');
-		if (server == NULL ||
-		    strcmp(server+1, client->proxy_address) == 0) {
-			if (server != NULL) {
-				if (server[-1] == ' ') server--;
-				*server = '\0';
-			}
-			if (*args == '\0') args = client->nick;
-			proxy_outdata(client, ":%s PONG proxy :%s\n", client->proxy_address, args);
+		/* Reply to PING, if the target parameter is either
+		   proxy_adress, our own nick or empty. */
+		char *params, *origin, *target;
+
+		params = event_get_params(args, 2, &origin, &target);
+		if (*target == '\0' ||
+		    g_strcasecmp(target, client->proxy_address) == 0 ||
+		    g_strcasecmp(target, client->nick) == 0) {
+			proxy_outdata(client, ":%s PONG %s :%s\n",
+				      client->proxy_address,
+                                      client->proxy_address, origin);
+			g_free(params);
 			return;
 		}
+		g_free(params);
 	}
-	if (strcmp(cmd, "NOTICE") == 0) {
-		char *str = g_strdup_printf("%s :\001IRSSILAG ", client->nick);
 
-		if (strncmp(args, str, strlen(str)) == 0)
-			proxy_outserver(client, "NOTICE %s", args);
-		g_free(str);
+	if (strcmp(cmd, "PROXY") == 0) {
+		if (g_strcasecmp(args, "CTCP ON") == 0) {
+                        /* client wants all ctcps */
+			ctcp_client = client;
+			proxy_outdata(client, ":%s NOTICE %s :You're now receiving all CTCPs sent to proxy\n",
+				      client->proxy_address, client->nick);
+		} else if (g_strcasecmp(args, "CTCP OFF") == 0) {
+                        /* client wants proxy to handle all ctcps */
+			ctcp_client = NULL;
+			proxy_outdata(client, ":%s NOTICE %s :Proxy is now handling itself all CTCPs\n",
+				      client->proxy_address, client->nick);
+		}
+		return;
 	}
 
 	if (client->server == NULL || !client->server->connected) {
-		proxy_outdata(client, ":%s NOTICE %s :Not connected to server",
+		proxy_outdata(client, ":%s NOTICE %s :Not connected to server\n",
 			      client->proxy_address, client->nick);
                 return;
 	}
 
-	server_handle = net_sendbuffer_handle(client->server->handle);
-	net_transmit(server_handle, cmd, strlen(cmd));
-	net_transmit(server_handle, " ", 1);
-	net_transmit(server_handle, args, strlen(args));
-	net_transmit(server_handle, "\n", 1);
-
+        /* check if the command could be redirected */
 	if (strcmp(cmd, "WHO") == 0)
 		grab_who(client, args);
 	else if (strcmp(cmd, "WHOIS") == 0) {
@@ -185,16 +198,11 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args)
 		for (p = args; *p != '\0'; p++)
 			if (*p == ',') *p = ' ';
 
-		proxy_redirect_event(client, args, 2,
-				     "event 318", -1, "event 402", -1,
-				     "event 401", 1, "event 311", 1,
-				     "event 301", 1, "event 312", 1,
-				     "event 313", 1, "event 317", 1,
-				     "event 319", 1, NULL);
+		proxy_redirect_event(client, "whois", 1, args, TRUE);
 	} else if (strcmp(cmd, "ISON") == 0)
-		proxy_redirect_event(client, NULL, 1, "event 303", -1, NULL);
+		proxy_redirect_event(client, "ison", 1, args, -1);
 	else if (strcmp(cmd, "USERHOST") == 0)
-		proxy_redirect_event(client, args, 1, "event 302", -1, "event 401", 1, NULL);
+		proxy_redirect_event(client, "userhost", 1, args, -1);
 	else if (strcmp(cmd, "MODE") == 0) {
 		/* convert dots to spaces */
 		char *slist, *str, mode, *p;
@@ -217,26 +225,16 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args)
 		str = g_strdup_printf("%s %s", args, slist);
 		switch (mode) {
 		case '\0':
-			while (argc-- > 0)
-				proxy_redirect_event(client, str, 3, "event 403", 1,
-						     "event 443", 1, "event 324", 1, NULL);
+			proxy_redirect_event(client, "mode channel", argc, str, -1);
 			break;
 		case 'b':
-			while (argc-- > 0)
-				proxy_redirect_event(client, str, 2, "event 403", 1,
-						     "event 368", 1, "event 367", 1, NULL);
+			proxy_redirect_event(client, "mode b", argc, str, -1);
 			break;
 		case 'e':
-			while (argc-- > 0)
-				proxy_redirect_event(client, str, 4, "event 403", 1,
-						     "event 482", 1, "event 472", -1,
-						     "event 349", 1, "event 348", 1, NULL);
+			proxy_redirect_event(client, "mode e", argc, str, -1);
 			break;
 		case 'I':
-			while (argc-- > 0)
-				proxy_redirect_event(client, str, 4, "event 403", 1,
-						     "event 482", 1, "event 472", -1,
-						     "event 347", 1, "event 346", 1, NULL);
+			proxy_redirect_event(client, "mode I", argc, str, -1);
 			break;
 		}
 		g_free(str);
@@ -248,10 +246,34 @@ static void handle_client_cmd(CLIENT_REC *client, char *cmd, char *args)
 		params = event_get_params(args, 2 | PARAM_FLAG_GETREST,
 					  &target, &msg);
 		proxy_outserver_all_except(client, "PRIVMSG %s", args);
-		signal_emit("message public", 5, client->server, msg,
-			    client->nick, client->proxy_address, target);
+
+		ignore_next = TRUE;
+		if (*msg != '\001' || msg[strlen(msg)-1] != '\001') {
+			signal_emit("message own_public", 4,
+				    client->server, msg, target, target);
+		} else if (strncmp(msg+1, "ACTION ", 7) == 0) {
+			/* action */
+                        msg[strlen(msg)-1] = '\0';
+			signal_emit("message irc own_action", 3,
+				    client->server, msg+8, target);
+		} else {
+                        /* CTCP */
+			char *p;
+
+			msg[strlen(msg)-1] = '\0';
+			p = strchr(msg, ' ');
+                        if (p != NULL) *p++ = '\0'; else p = "";
+
+			signal_emit("message irc own_ctcp", 4,
+				    client->server, msg+1, p, target);
+		}
+		ignore_next = FALSE;
 		g_free(params);
+	} else if (strcmp(cmd, "PING") == 0) {
+		proxy_redirect_event(client, "ping", 1, NULL, TRUE);
 	}
+
+	irc_send_cmd(client->server, data);
 }
 
 static void sig_listen_client(CLIENT_REC *client)
@@ -279,7 +301,7 @@ static void sig_listen_client(CLIENT_REC *client)
 		if (*args == ':') args++;
 		g_strup(cmd);
 
-		handle_client_cmd(client, cmd, args);
+		handle_client_cmd(client, cmd, args, str);
 
 		g_free(cmd);
 	}
@@ -304,14 +326,24 @@ static void sig_listen(LISTEN_REC *listen)
 	rec = g_new0(CLIENT_REC, 1);
 	rec->listen = listen;
 	rec->handle = handle;
-	rec->proxy_address = g_strdup(listen->ircnet);
-	rec->server = IRC_SERVER(server_find_chatnet(listen->ircnet));
+        rec->host = g_strdup(host);
+	if (strcmp(listen->ircnet, "*") == 0) {
+		rec->proxy_address = g_strdup("irc.proxy");
+		rec->server = servers == NULL ? NULL : IRC_SERVER(servers->data);
+	} else {
+		rec->proxy_address = g_strdup_printf("%s.proxy", listen->ircnet);
+		rec->server = servers == NULL ? NULL :
+			IRC_SERVER(server_find_chatnet(listen->ircnet));
+	}
 	rec->tag = g_input_add(handle, G_INPUT_READ,
 			       (GInputFunction) sig_listen_client, rec);
 
-	proxy_clients = g_slist_append(proxy_clients, rec);
+	proxy_clients = g_slist_prepend(proxy_clients, rec);
+	rec->listen->clients = g_slist_prepend(rec->listen->clients, rec);
+
+        signal_emit("proxy client connected", 1, rec);
 	printtext(NULL, NULL, MSGLEVEL_CLIENTNOTICE,
-		  "Proxy: Client connected from %s", host);
+		  "Proxy: Client connected from %s", rec->host);
 }
 
 static void sig_incoming(IRC_SERVER_REC *server, const char *line)
@@ -325,8 +357,10 @@ static void sig_incoming(IRC_SERVER_REC *server, const char *line)
 static void sig_server_event(IRC_SERVER_REC *server, const char *line,
 			     const char *nick, const char *address)
 {
-	GSList *list;
+        void *client;
+        const char *signal;
 	char *event, *args;
+        int redirected;
 
 	g_return_if_fail(line != NULL);
 	if (!IS_IRC_SERVER(server))
@@ -337,39 +371,47 @@ static void sig_server_event(IRC_SERVER_REC *server, const char *line,
 	args = strchr(event+6, ' ');
 	if (args != NULL) *args++ = '\0'; else args = "";
 	while (*args == ' ') args++;
+        g_strdown(event);
 
-	list = server_redirect_getqueue(SERVER(server), event, args);
-
-	if (list != NULL) {
+	signal = server_redirect_peek_signal(server, event, args, &redirected);
+	if ((signal != NULL && strncmp(signal, "proxy ", 6) != 0) ||
+	    (signal == NULL && redirected)) {
 		/* we want to send this to one client (or proxy itself) only */
-		REDIRECT_REC *rec;
-		void *handle;
+		/* proxy only */
+		g_free(event);
+		return;
+	}
 
-		rec = list->data;
-		if (g_strncasecmp(rec->name, "proxy ", 6) != 0) {
-			/* proxy only */
-			g_free(event);
-			return;
-		}
-
-		if (sscanf(rec->name+6, "%p", &handle) == 1) {
+	if (signal != NULL) {
+                server_redirect_get_signal(server, event, args);
+		if (sscanf(signal+6, "%p", &client) == 1) {
 			/* send it to specific client only */
-			server_redirect_remove_next(SERVER(server), event, list);
-			net_transmit(handle, next_line->str, next_line->len);
+			if (g_slist_find(proxy_clients, client) != NULL)
+				net_transmit(((CLIENT_REC *) client)->handle, next_line->str, next_line->len);
 			g_free(event);
                         signal_stop();
 			return;
 		}
 	}
 
-	if (g_strcasecmp(event, "event ping") == 0 ||
-	    (g_strcasecmp(event, "event privmsg") == 0 &&
-	     strstr(args, " :\001") != NULL &&
-	     strstr(args, " :\001ACTION") == NULL) ||
-	    (g_strcasecmp(event, "event notice") == 0 &&
-	     strstr(args, " :\001IRSSILAG") != NULL)) {
-		/* We want to answer ourself to PINGs and CTCPs,
-		   also don't let clients see replies to IRSSILAG requests */
+        if (strcmp(event, "event privmsg") == 0 &&
+	    strstr(args, " :\001") != NULL &&
+	    strstr(args, " :\001ACTION") == NULL) {
+		/* CTCP - either answer ourself or forward it to one client */
+		if (ctcp_client != NULL) {
+                        /* one of the clients answers */
+			net_transmit(ctcp_client->handle,
+				     next_line->str, next_line->len);
+			signal_stop();
+		}
+		g_free(event);
+		return;
+	}
+
+	if (strcmp(event, "event ping") == 0 ||
+	    strcmp(event, "event pong") == 0) {
+		/* We want to answer ourself to PINGs and CTCPs.
+		   Also hide PONGs from clients. */
 		g_free(event);
 		return;
 	}
@@ -383,18 +425,23 @@ static void sig_server_event(IRC_SERVER_REC *server, const char *line,
 static void event_connected(IRC_SERVER_REC *server)
 {
 	GSList *tmp;
+        const char *chatnet;
 
-	if (!IS_IRC_SERVER(server) || server->connrec->chatnet == NULL)
+	if (!IS_IRC_SERVER(server))
 		return;
 
+        chatnet = server->connrec->chatnet;
 	for (tmp = proxy_clients; tmp != NULL; tmp = tmp->next) {
 		CLIENT_REC *rec = tmp->data;
 
 		if (rec->connected && rec->server == NULL &&
-		    g_strcasecmp(server->connrec->chatnet, rec->listen->ircnet) == 0) {
-			proxy_outdata(rec, ":%s NOTICE %s :Connected to server",
+		    (strcmp(rec->listen->ircnet, "*") == 0 ||
+		     (chatnet != NULL &&
+		      g_strcasecmp(chatnet, rec->listen->ircnet) == 0))) {
+			proxy_outdata(rec, ":%s NOTICE %s :Connected to server\n",
 				      rec->proxy_address, rec->nick);
 			rec->server = server;
+			proxy_client_reset_nick(rec);
 		}
 	}
 }
@@ -438,6 +485,9 @@ static void event_nick(IRC_SERVER_REC *server, const char *data,
 {
 	GSList *tmp;
 
+	if (!IS_IRC_SERVER(server))
+		return;
+
 	if (g_strcasecmp(orignick, server->nick) != 0)
 		return;
 
@@ -452,6 +502,26 @@ static void event_nick(IRC_SERVER_REC *server, const char *data,
 	}
 }
 
+static void sig_message_own_public(IRC_SERVER_REC *server, const char *msg,
+                                   const char *target)
+{
+	if (!IS_IRC_SERVER(server))
+		return;
+
+	if (!ignore_next)
+		proxy_outserver_all(server, "PRIVMSG %s :%s", target, msg);
+}
+
+static void sig_message_own_action(IRC_SERVER_REC *server, const char *msg,
+                                   const char *target)
+{
+	if (!IS_IRC_SERVER(server))
+		return;
+
+	if (!ignore_next)
+		proxy_outserver_all(server, "PRIVMSG %s :\001ACTION %s\001", target, msg);
+}
+
 static LISTEN_REC *find_listen(const char *ircnet, int port)
 {
 	GSList *tmp;
@@ -470,19 +540,38 @@ static LISTEN_REC *find_listen(const char *ircnet, int port)
 static void add_listen(const char *ircnet, int port)
 {
 	LISTEN_REC *rec;
+	IPADDR ip4, ip6, *my_ip;
 
-	if (port <= 0 || *ircnet == '\0') return;
+	if (port <= 0 || *ircnet == '\0')
+		return;
+
+	/* bind to specific host/ip? */
+	my_ip = NULL;
+	if (*settings_get_str("irssiproxy_bind") != '\0') {
+		if (net_gethostbyname(settings_get_str("irssiproxy_bind"),
+				      &ip4, &ip6) != 0) {
+			printtext(NULL, NULL, MSGLEVEL_CLIENTERROR,
+				  "Proxy: can not resolve '%s' - aborting",
+				  settings_get_str("irssiproxy_bind"));
+			return;
+		}
+
+		my_ip = ip6.family == 0 ? &ip4 : ip4.family == 0 ||
+			settings_get_bool("resolve_prefer_ipv6") ? &ip6 : &ip4;
+	}
 
 	rec = g_new0(LISTEN_REC, 1);
 	rec->ircnet = g_strdup(ircnet);
 	rec->port = port;
 
-	/* start listening */
-	rec->handle = net_listen(NULL, &rec->port);
+	rec->handle = net_listen(my_ip, &rec->port);
+
 	if (rec->handle == NULL) {
 		printtext(NULL, NULL, MSGLEVEL_CLIENTERROR,
 			  "Proxy: Listen in port %d failed: %s",
 			  rec->port, g_strerror(errno));
+		g_free(rec->ircnet);
+                g_free(rec);
 		return;
 	}
 
@@ -496,6 +585,9 @@ static void remove_listen(LISTEN_REC *rec)
 {
 	proxy_listens = g_slist_remove(proxy_listens, rec);
 
+	while (rec->clients != NULL)
+		remove_client(rec->clients->data);
+
 	net_disconnect(rec->handle);
 	g_source_remove(rec->tag);
 	g_free(rec->ircnet);
@@ -537,7 +629,7 @@ static void read_settings(void)
 	}
 }
 
-void plugin_proxy_listen_init(void)
+void proxy_listen_init(void)
 {
 	next_line = g_string_new(NULL);
 
@@ -550,13 +642,13 @@ void plugin_proxy_listen_init(void)
 	signal_add("event connected", (SIGNAL_FUNC) event_connected);
 	signal_add("server disconnected", (SIGNAL_FUNC) sig_server_disconnected);
 	signal_add("event nick", (SIGNAL_FUNC) event_nick);
+	signal_add("message own_public", (SIGNAL_FUNC) sig_message_own_public);
+	signal_add("message irc own_action", (SIGNAL_FUNC) sig_message_own_action);
 	signal_add("setup changed", (SIGNAL_FUNC) read_settings);
 }
 
-void plugin_proxy_listen_deinit(void)
+void proxy_listen_deinit(void)
 {
-	while (proxy_clients != NULL)
-		remove_client(proxy_clients->data);
 	while (proxy_listens != NULL)
 		remove_listen(proxy_listens->data);
 	g_string_free(next_line, TRUE);
@@ -566,5 +658,7 @@ void plugin_proxy_listen_deinit(void)
 	signal_remove("event connected", (SIGNAL_FUNC) event_connected);
 	signal_remove("server disconnected", (SIGNAL_FUNC) sig_server_disconnected);
 	signal_remove("event nick", (SIGNAL_FUNC) event_nick);
+	signal_remove("message own_public", (SIGNAL_FUNC) sig_message_own_public);
+	signal_remove("message irc own_action", (SIGNAL_FUNC) sig_message_own_action);
 	signal_remove("setup changed", (SIGNAL_FUNC) read_settings);
 }
