@@ -4,6 +4,7 @@
 
 #include "network.h"
 #include "line-split.h"
+#include "irc.h"
 #include "irc-servers.h"
 
 typedef struct {
@@ -12,12 +13,14 @@ typedef struct {
 
 	int tag;
 	GIOChannel *handle;
+
+	GSList *clients;
 } LISTEN_REC;
 
 typedef struct {
 	LINEBUF_REC *buffer;
 
-	char *nick;
+	char *nick, *host;
 	GIOChannel *handle;
 	int tag;
 
@@ -25,24 +28,23 @@ typedef struct {
 	LISTEN_REC *listen;
 	IRC_SERVER_REC *server;
 	unsigned int pass_sent:1;
+	unsigned int user_sent:1;
 	unsigned int connected:1;
 } CLIENT_REC;
 
 extern GSList *proxy_listens;
 extern GSList *proxy_clients;
 
-void plugin_proxy_setup_init(void);
-void plugin_proxy_setup_deinit(void);
-
-void plugin_proxy_listen_init(void);
-void plugin_proxy_listen_deinit(void);
+void proxy_listen_init(void);
+void proxy_listen_deinit(void);
 
 void proxy_settings_init(void);
 
-void plugin_proxy_dump_data(CLIENT_REC *client);
+void proxy_dump_data(CLIENT_REC *client);
+void proxy_client_reset_nick(CLIENT_REC *client);
 
 void proxy_outdata(CLIENT_REC *client, const char *data, ...);
 void proxy_outdata_all(IRC_SERVER_REC *server, const char *data, ...);
 void proxy_outserver(CLIENT_REC *client, const char *data, ...);
-/*void proxy_outserver_all(const char *data, ...);*/
+void proxy_outserver_all(IRC_SERVER_REC *server, const char *data, ...);
 void proxy_outserver_all_except(CLIENT_REC *client, const char *data, ...);
